# Copyright (C) 2008-2009 Open Society Institute
#               Thomas Moroz: tmoroz@sorosny.org
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License Version 2 as published
# by the Free Software Foundation.  You may not use, modify or distribute
# this program under any other version of the GNU General Public License.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import unittest

from zope.testing.cleanup import cleanUp
from zope.interface import implements
from zope.interface import alsoProvides

from repoze.bfg import testing

from karl.models.interfaces import ISite

from karl.content.interfaces import ICommunityFolder
from karl.models.interfaces import IIntranets
from karl.content.interfaces import IIntranetFolder


class TestFolderCustomizer(unittest.TestCase):
    def setUp(self):
        cleanUp()

    def tearDown(self):
        cleanUp()

    def _getTargetClass(self):
        from osi.views.adapters import FolderCustomizer
        return FolderCustomizer

    def _makeOne(self, context, request):
        return self._getTargetClass()(context, request)

    def test_class_conforms_to_IFolderCustomizer(self):
        from zope.interface.verify import verifyClass
        from karl.content.views.interfaces import IFolderCustomizer
        verifyClass(IFolderCustomizer, self._getTargetClass())

    def test_markers_default(self):
        context = DummyFolder()
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        self.assertEqual(adapter.markers, [])

    def test_markers_intranetfolder(self):
        intranets = DummyIntranets()
        context = DummyFolder()
        intranets['folder'] = context
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        self.assertEqual(adapter.markers, [IIntranetFolder])


class TestShowSendalert(unittest.TestCase):
    def setUp(self):
        cleanUp()

    def tearDown(self):
        cleanUp()

    def _getTargetClass(self):
        from osi.views.adapters import ShowSendalert
        return ShowSendalert

    def _makeOne(self, context, request):
        return self._getTargetClass()(context, request)

    def test_class_conforms_to_IShowSendalert(self):
        from zope.interface.verify import verifyClass
        from karl.content.views.interfaces import IShowSendalert
        verifyClass(IShowSendalert, self._getTargetClass())

    def test_not_intranet(self):
        context = DummyFolder()
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        self.assertEqual(adapter.show_sendalert, True)

    def test_intranet(self):
        intranets = DummyIntranets()
        context = DummyFolder()
        intranets['folder'] = context
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        self.assertEqual(adapter.show_sendalert, False)


class TestLayoutProvider(unittest.TestCase):
    def setUp(self):
        cleanUp()

    def tearDown(self):
        cleanUp()

    def _getTargetClass(self):
        from osi.views.adapters import LayoutProvider
        return LayoutProvider

    def _makeOne(self, context, request):
        return self._getTargetClass()(context, request)

    def test_class_conforms_to_ILayoutProvider(self):
        from zope.interface.verify import verifyClass
        from karl.views.interfaces import ILayoutProvider
        verifyClass(ILayoutProvider, self._getTargetClass())

    def test_unknown_layout(self):
        # No interface on context, no default layout passed in
        context = DummyFolder()
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        layout = adapter()
        self.assertEqual(layout, None)

    def test_default_is_community(self):
        # No interface on context, pass in community as choice
        context = DummyFolder()
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        layout = adapter('community')
        self.assert_('community_layout.pt' in layout.filename)

    def test_default_is_generic(self):
        # No interface on context, pass in community as choice
        context = DummyFolder()
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        layout = adapter('generic')
        self.assert_('generic_layout.pt' in layout.filename)

    def test_forums_outside_intranet(self):
        from karl.content.interfaces import IForum
        context = DummyFolder()
        alsoProvides(context, IForum)
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        # The karl.content.views.forums code will ask for community
        # layout (to be generic outside an intranet).  In this test,
        # we are NOT inside an intranet, causing the OSI policy (use
        # generic layout) NOT to be triggered.
        layout = adapter('community')
        self.assert_('community_layout.pt' in layout.filename)

    def test_forums_inside_intranet(self):
        from karl.content.interfaces import IForum
        from karl.models.interfaces import IIntranet
        context = DummyFolder()
        alsoProvides(context, IForum)
        alsoProvides(context, IIntranet)
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        # The karl.content.views.forums code will ask for community
        # layout (to be generic outside an intranet), but the OSI
        # policy will impose a different o-wrap because we are in an
        # intranet.
        layout = adapter('community')
        self.assert_('generic_layout.pt' in layout.filename)

    def test_calendarevent_in_networkevents(self):
        # A "network event" is really a calendar event.  It exists at
        # the /offices/network-events level.
        from karl.content.interfaces import ICalendarEvent
        from karl.models.interfaces import IIntranets
        context = DummyFolder()
        alsoProvides(context, ICalendarEvent)
        alsoProvides(context, IIntranets)
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        layout = adapter('community')
        self.assert_('generic_layout.pt' in layout.filename)

    def test_network_events(self):
        # A "network event" is really a calendar event.  It exists at
        # the /offices/network-events level.
        from karl.content.views.interfaces import INetworkEventsMarker
        from karl.models.interfaces import IIntranets
        context = DummyFolder()
        alsoProvides(context, INetworkEventsMarker)
        alsoProvides(context, IIntranets)
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        layout = adapter('community')
        self.assert_('generic_layout.pt' in layout.filename)

class TestInvitationBoilerplate(unittest.TestCase):
    def setUp(self):
        cleanUp()

    def tearDown(self):
        cleanUp()

    def _getTargetClass(self):
        from osi.views.adapters import InvitationBoilerplate
        return InvitationBoilerplate

    def _makeOne(self, context, request):
        return self._getTargetClass()(context, request)

    def test_class_conforms_to_IInvitationBoilerplate(self):
        from zope.interface.verify import verifyClass
        from karl.views.interfaces import IInvitationBoilerplate
        verifyClass(IInvitationBoilerplate, self._getTargetClass())

    def test_terms_and_conditions(self):
        from karl.views.interfaces import IInvitationBoilerplate
        context = DummyFolder()
        alsoProvides(context, IInvitationBoilerplate)
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        self.assertEqual(adapter.terms_and_conditions[0:7], '<p>No t')

    def test_privacy_statement(self):
        from karl.views.interfaces import IInvitationBoilerplate
        context = DummyFolder()
        alsoProvides(context, IInvitationBoilerplate)
        request = testing.DummyRequest()
        adapter = self._makeOne(context, request)
        self.assertEqual(adapter.privacy_statement[0:7], '<p>No p')

class TestOSIFooter(unittest.TestCase):

    def _getTargetClass(self):
        from osi.views.adapters import OSIFooter
        return OSIFooter

    def _makeOne(self, context=None, request=None):
        if context is None:
            context = testing.DummyModel()
        if request is None:
            request = testing.DummyRequest()
        return self._getTargetClass()(context, request)

    def test_class_conforms_to_IFooter(self):
        from zope.interface.verify import verifyClass
        from karl.views.interfaces import IFooter
        verifyClass(IFooter, self._getTargetClass())

    def test_instance_conforms_to_IFooter(self):
        from zope.interface.verify import verifyObject
        from karl.views.interfaces import IFooter
        verifyObject(IFooter, self._makeOne())

    def test_it(self):
        renderer = testing.registerDummyRenderer('templates/footer.pt')
        footer = self._makeOne()
        api = object()
        response = footer(api)
        self.failUnless(renderer.api is api)


class DummySearchAdapter:
    def __init__(self, context):
        self.context = context

    def __call__(self, **kw):
        return 0, [], None

class DummySite(testing.DummyModel):
    title = 'dummysite'
    implements(ISite)

class DummyIntranets(testing.DummyModel):
    title = 'dummyintranets'
    implements(IIntranets)

class DummyFolder(testing.DummyModel):
    title = 'dummyfolder'
    implements(ICommunityFolder)

class DummyToolFactory:
    def __init__(self, present=False):
        self.present = present

    def add(self, context, request):
        self.added = True

    def remove(self, context, request):
        self.removed = True

    def is_present(self, context, request):
        return self.present

